use pool::Pool;

struct Node<T> {
    pub data:T,
    pub prev:Option<usize>,
    pub next:Option<usize>,
}

pub struct Chain<T> {
    nodes:Pool<Node<T>>,
    begin:Option<usize>,
    end:Option<usize>,
}
impl<T> Chain<T> {
    pub fn new() -> Self
    {
        Self {
            nodes:Pool::new(),
            begin:None,
            end:None,
        }
    }

    pub fn add(&mut self, data:T) -> usize
    // Add an element to the top of the chain.
    //
    {
        let id = self.nodes.add(Node {
            data:data,
            prev:None,
            next:self.begin,
        });
        self.begin = Some(id);
        id
    }

    pub fn promote(&mut self, id:usize)
    // Move an element to the top of the chain.
    //
    {
        let mut prev = None;
        let mut next = None;

        if let Some(node) = self.nodes.get_mut(id) {
            prev = node.prev;
            next = node.next;
            node.prev = None;
            node.next = self.begin;
        }
        self.begin = Some(id);
        if self.begin == self.end { self.end = next; }

        if let Some(pid) = prev {
            if let Some(pnode) = self.nodes.get_mut(pid) {
                pnode.next = next;
            }
        }
        if let Some(nid) = next {
            if let Some(nnode) = self.nodes.get_mut(nid) {
                nnode.next = prev;
            }
        }
    }

    pub fn get(&self, id:usize) -> Option<&T>
    {
        if let Some(node) = self.nodes.get(id) {
            Some(&node.data)
        } else {
            None
        }
    }

    pub fn set(&mut self, id:usize, data:T)
    {
        if let Some(node) = self.nodes.get_mut(id) {
            node.data = data;
        }
    }

    pub fn begin(&self) -> Option<usize>
    {
        self.begin
    }

    pub fn next(&self, id:usize) -> Option<usize>
    {
        if let Some(node) = self.nodes.get(id) {
            node.next
        } else {
            None
        }
    }

    pub fn remove(&mut self)
    // Remove the last element.
    //
    {
        let mut prev = None;
        if let Some(end) = self.end {
            if let Some(node) = self.nodes.get_mut(end) {
                prev = node.prev;
            }
            self.nodes.remove(end).ok();

            if let Some(id) = prev {
                if let Some(node) = self.nodes.get_mut(id) {
                    node.next = None;
                }
            }
            self.end = prev;
        }
    }
}
